%model=rss(4,1,1);
%load model_AW1

model=ss(...
    [-1.364   0.4693    0.736    1.131
    -1.08   -1.424   0.1945  -0.7132
    0.0499   0.8704  -0.9675  -0.3388
    -0.9333   0.8579  -0.5436  -0.9997],...
    [  0.05574; 0; -0.04123; -1.128],...
    [-1.349       0  0.9535  0.1286],...
    0);

ts=.5; % sampling time

sysd=c2d(model,ts);

[A,B,C,D]=ssdata(sysd);

% Controllore
policontinuo=[-2+j, -2-j -5 -5];
polidiscreto=exp(ts*policontinuo);
K=-acker(A,B,polidiscreto);

% Osservatore
policontinuo=[-10,-10,-10,-10];
polidiscreto=exp(ts*policontinuo);
L=acker(A',C',polidiscreto)';

H=1/dcgain(ss(A+B*K,B,C,D,ts));

% Saturazione
umin=-2;
umax=2;

% Simulazione senza anti-windup
x=[0;0;0;0];
xhat=[0;0;0;0];
u=0;
r=1;
XX=[];
YY=[];
UU=[];

T=100;
time=(0:T-1)'*ts;

for t=1:T
    v=K*xhat+H*r;
    
    % Saturazione: u=sat(v)
    if v<umin,
        u=umin;
    elseif v>umax,
        u=umax;
    else 
        u=v;
    end
    
    y=C*x+D*u;
    
    % Registra variabili
    XX=[XX;x'];
    YY=[YY;y];
    UU=[UU;u];
    
    % Update
    xhat=A*xhat+L*(y-C*xhat)+B*v;  % Senza anti-windup
    
    x=A*x+B*u;
end


% Simulazione con anti-windup
x=[0;0;0;0];
xhat=[0;0;0;0];
u=0;
r=1;
XXaw=[];
YYaw=[];
UUaw=[];

T=100;
time=(0:T-1)'*ts;

for t=1:T
    v=K*xhat+H*r;
    
    % Saturazione: u=sat(v)
    if v<umin,
        u=umin;
    elseif v>umax,
        u=umax;
    else 
        u=v;
    end
    
    y=C*x+D*u;
    
    % Registra variabili
    XXaw=[XXaw;x'];
    YYaw=[YYaw;y];
    UUaw=[UUaw;u];
    
    % Update
    xhat=A*xhat+L*(y-C*xhat)+B*u;  % Con anti-windup
   
    x=A*x+B*u;
end



subplot(211);
h=stairs(time,YY);
set(h,'LineWidth',1.5,'LineStyle','--');
hold on
h=stairs(time,YYaw);
set(h,'LineWidth',3.0,'color',[.7 .2 .2]);
h=ylabel('Uscita y(t)');
set(h,'FontName','cmssq8','FontSize',14);
grid
subplot(212);
h=stairs(time,UU);
set(h,'LineWidth',1.5,'LineStyle','--');
hold on
h=stairs(time,UUaw);
set(h,'LineWidth',3.0,'color',[.7 .2 .2]);
h=ylabel('Ingresso u(t)');
set(h,'FontName','cmssq8','FontSize',14);
h=xlabel('tempo t');
set(h,'FontName','cmssq8','FontSize',14);
grid
